Package: Biostrings
Title: Efficient manipulation of biological strings
Description: Memory efficient string containers, string matching
        algorithms, and other utilities, for fast manipulation of large
        biological sequences or sets of sequences.
biocViews: SequenceMatching, Alignment, Sequencing, Genetics,
        DataImport, DataRepresentation, Infrastructure
URL: https://bioconductor.org/packages/Biostrings
BugReports: https://github.com/Bioconductor/Biostrings/issues
Version: 2.81.6
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Hervé", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"),
	person("Patrick", "Aboyoun", role="aut"),
	person("Robert", "Gentleman", role="aut"),
	person("Saikat", "DebRoy", role="aut"),
	person("Vince", "Carey", role="ctb"),
	person("Nicolas", "Delhomme", role="ctb"),
	person("Felix", "Ernst", role="ctb"),
	person("Wolfgang", "Huber", role="ctb",
	       comment="'matchprobes' vignette"),
	person("Haleema", "Khan", role="ctb",
	       comment="Converted 'matchprobes' vignette from Sweave to RMarkdown"),
	person("Aidan", "Lakshman", role="ctb"),
	person("Michael", "Lawrence", role="ctb"),
	person("Kieran", "O'Neill", role="ctb"),
	person("Valerie", "Obenchain", role="ctb"),
	person("Marcel", "Ramos", role="ctb"),
	person("Albert", "Vill", role="ctb"),
	person("Jen", "Wokaty", role="ctb",
	       comment="Converted 'matchprobes' vignette from Sweave to RMarkdown"),
	person("Erik", "Wright", role="ctb"))
Depends: R (>= 4.1.0), BiocGenerics (>= 0.37.0), S4Vectors (>=
        0.27.12), IRanges (>= 2.31.2), XVector (>= 0.37.1), Seqinfo
Imports: methods, grDevices, stats, crayon
LinkingTo: S4Vectors, IRanges, XVector
Suggests: utils, graphics, pwalign, MultipleAlignment, BSgenome (>=
        1.13.14), BSgenome.Celegans.UCSC.ce2 (>= 1.3.11),
        BSgenome.Dmelanogaster.UCSC.dm3 (>= 1.3.11),
        BSgenome.Hsapiens.UCSC.hg18, drosophila2probe, hgu95av2probe,
        hgu133aprobe, GenomicFeatures (>= 1.3.14), hgu95av2cdf, affy
        (>= 1.41.3), affydata (>= 1.11.5), RUnit, BiocStyle, knitr,
        testthat (>= 3.0.0), covr
VignetteBuilder: knitr
Collate: utils.R IUPAC_CODE_MAP.R AMINO_ACID_CODE.R GENETIC_CODE.R
        XStringCodec-class.R seqtype.R coloring.R XString-class.R
        XStringSet-class.R XStringSet-comparison.R XStringViews-class.R
        MaskedXString-class.R XStringSetList-class.R seqinfo-methods.R
        xscat.R XStringSet-io.R letter.R getSeq.R letterFrequency.R
        dinucleotideFrequencyTest.R chartr.R reverseComplement.R
        translate.R toComplex.R replaceAt.R replaceLetterAt.R
        injectHardMask.R padAndClip.R strsplit-methods.R misc.R
        MIndex-class.R MIndexList-class.R lowlevel-matching.R
        match-utils.R matchPattern.R maskMotif.R matchLRPatterns.R
        trimLRPatterns.R matchProbePair.R matchPWM.R findPalindromes.R
        PDict-class.R matchPDict.R XStringQuality-class.R
        QualityScaledXStringSet.R lcsuffix.R MultipleAlignment.R
        longestConsecutive.R predefined_scoring_matrices.R zzz.R
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-07-29 20:22:09 UTC
RemoteUrl: https://github.com/bioc/Biostrings
RemoteRef: HEAD
RemoteSha: bf137292de3dbee52700221067f0170522b9b09f
NeedsCompilation: yes
Packaged: 2026-07-31 06:16:50 UTC; root
Author: Hervé Pagès [aut, cre],
  Patrick Aboyoun [aut],
  Robert Gentleman [aut],
  Saikat DebRoy [aut],
  Vince Carey [ctb],
  Nicolas Delhomme [ctb],
  Felix Ernst [ctb],
  Wolfgang Huber [ctb] ('matchprobes' vignette),
  Haleema Khan [ctb] (Converted 'matchprobes' vignette from Sweave to
    RMarkdown),
  Aidan Lakshman [ctb],
  Michael Lawrence [ctb],
  Kieran O'Neill [ctb],
  Valerie Obenchain [ctb],
  Marcel Ramos [ctb],
  Albert Vill [ctb],
  Jen Wokaty [ctb] (Converted 'matchprobes' vignette from Sweave to
    RMarkdown),
  Erik Wright [ctb]
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
Built: R 4.6.1; aarch64-apple-darwin23; 2026-07-31 06:36:02 UTC; unix
