[<img src="https://github.com/Bioconductor/BiocStickers/blob/devel/Biostrings/Biostrings.png" width="135" align="left">](https://bioconductor.org/packages/Biostrings)

[<img src="https://bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="180" align="right">](https://bioconductor.org/)

**Biostrings** is an R/Bioconductor package for efficient manipulation of biological strings.

See https://bioconductor.org/packages/Biostrings for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

<sub>_Biostrings sticker courtesy of [Roberto Bonelli](https://github.com/Robbie90)._</sub>
