
print.moved_to_pwalign_pkg <- function(x, ...)
    warning(wmsg("all the BLOSUM* and PAM* scoring matrices are now located ",
                 "in the pwalign package and will soon be removed from the ",
                 "Biostrings package"))

