---
title: "MultipleAlignment Objects"

vignette: >
  %\VignetteIndexEntry{Multiple Alignments}
  %\VignetteKeywords{DNA, RNA, Sequence, Biostrings, Sequence alignment}
  %\VignettePackage{Biostrings}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
output:
  BiocStyle::html_document:
        number_sections: true
        toc: true
        toc_depth: 4
editor_options: 
  markdown: 
    wrap: 72
---

IMPORTANT NOTE:

Starting with `r Biocpkg("Biostrings")` 2.81.3 (Bioconductor 3.24),
the "MultipleAlignment Objects" vignette is located in the
`r Biocpkg("MultipleAlignment")` package.

Install the `r Biocpkg("MultipleAlignment")` package with:
```{r, eval=FALSE}
if (!require("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("MultipleAlignment")
```

and access the "MultipleAlignment Objects" vignette with:
```{r, eval=FALSE}
browseVignettes("MultipleAlignment")
```

